module Simple::Sharding
  class ConnectionHandler
    # check
    def check(shard_id)
      shard_id.present?
    end

    # establish all connection
    def connect_all
      Core.config.keys.each do |shard_id|
        connect(shard_id)
      end
    end

    def connect(shard_id)
      resolver = ActiveRecord::ConnectionAdapters::ConnectionSpecification::Resolver.new(Config.config(shard_id))
      connect_spec = resolver.spec(shard_id.to_sym)
      connect_spec
    end

    def self.setup
      @@connection_handler = ActiveRecord::ConnectionAdapters::ConnectionHandler.new
      @@connection_pool_owners = {}
    end

    def self.connection_handler
      raise 'Shards::ConnectionHandler was not setup' unless defined? @@connection_handler
      @@connection_handler
    end

    def self.connection_pool(shard_id)
      connection_handler.retrieve_connection_pool(connection_pool_owner(shard_id))
    end

    def self.retrie_connection(shard_id)
      connection_handler.retrieve_connection(connection_pool_owner(shard_id))
    end

    def self.connected?(shard_id)
      connection_handler.connected?(connection_pool_owner(shard_id))
    end

    def self.remove_connection(shard_id)
      connection_handler.remove_connection(connection_pool_owner(shard_id))
    end

    def self.connection_pool_owner(shard_id)
      @@connection_pool_owners[shard_id] ||= ConnectionPoolOwner.new(shard_id)
    end

    # Connection pool need a class to own each connection
    class ConnectionPoolOwner
      attr_reader :name

      def initialize(name)
        @name = name
      end

      # Safeguard in case pool cannot be retrieved for owner. This makes the error clear
      def superclass
        raise Exception, "ConnectionPool could not be retrieved for #{self}. "
      end

      # in case owner ends up printed by rails in an error message when retrieving connection
      def to_s
        "ConnectionPoolOwner with name #{self.name}"
      end
    end
  end


end