require 'simple/sharding/active_record_extensions'
require 'simple/sharding/config'
require 'simple/sharding/connection_handler'
require 'simple/sharding/shard_thread_registry'

module Simple::Sharding
  class Core
    # block support
    def self.sharding(shard_id)
      begin
        raise "no shard group #{shard_id} found in pool." if ConnectionHandler.check(shard_id)
        ShardThreadRegistry.push_current_shard(shard_id)
        yield
      ensure
        # Releases connections in case user left some connection in the reserved state
        # (by calling retrieve_connection instead of with_connection). Also, using
        # normal activerecord queries leaves a connection in the reserved state
        # Obs: don't do this with a master database connection
        ConnectionHandler.connection_pool(shard_id).release_connection if !shard_id.blank?
      end

    end

    # configuration
    def self.config(env=Rails.env)
      @@db_config ||= YAML.load_file(Config.config(:shard_config_file))
      env_config = @@db_config[env]
      return env_config if env_config

      # error
      raise "No correct config for env: #{env}."
    end

    def setup
      if block_given?
        yield Config
      end

      ConnectionHandler.connect_all
      ActiveRecordExtensions.extend_active_record_scope
    end
  end
end
